/*
    Copyright (C) 2013 Lukasz Nidecki SQ5RWU
	
    This file is part of ArduinoQAPRS.

    ArduinoQAPRS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ArduinoQAPRS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ArduinoQAPRS; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /* Erweiterung als WX Station mit Temperatursensor für Arduino Nano; DL1STL, 19.11.2015 
    Projektname: Arduino APRS Modem
    PCB Layout: V4; vom 3.2.2016
    Beachte: Zum Programmieren den Anschluss des GPS Moduls trennen ! 
    
 */ 
   
 #include <string.h>
 #include <Arduino.h>
 #include <ArduinoQAPRS.h>
 #include <SPI.h>
 
 #include <Wire.h>
 #include <Adafruit_BMP085.h>     // temperature sensor BMP085 by I2C

char * packet_buffer  = "                                                                    \n ";
char floatbuf[32];
char gradbuf[4];
char from_addr[] = "DL1xy-9 ";      // change according to your needs
char dest_addr[] = "APRS  ";     // change according to your needs
char relays[] = "WIDE2 1";       // change according to your needs 

String lat,lon,ns,ew;
String aprs = "";
String GPSStringbuf = "";
String GPSString = "";
int ledPin = 13;                  // LED test pin
int status,alti; 
boolean stringComplete = false;

Adafruit_BMP085 bmp;

// Pin 13 has an LED connected on most Arduino boards.
int led = 13;

///////////////////////////////////////////////////////// 
void setup()
{
  Serial.begin(9600);            // data rate seriall communication at RX0/TX0
  Serial.println("my_QARPS_GPS; Extended ARPS Modem with WX data over I2C");
  if (!bmp.begin()) {
	Serial.println("Could not find a valid BMP085 sensor, check wiring!");
	while (1) {}
  } else {
	Serial.println("Found BMP085 or BMP180 sensor. ");
  }    

  Serial.println("WAIT!");
  delay(100);
  pinMode(ledPin, OUTPUT);       // initialize LED pin
  GPSString.reserve(100);        // reserving 100 characters

// initialize the digital pin as an output.
  pinMode(led, OUTPUT);    
 
  // pin 3 to sensePin 
  // pin 2 to txPin [wyjscie] 
  QAPRS.init(3,2);
  delay(2000); 
  
}

/////////////////////////////////////////////////////////// 
void loop()
{
  while (Serial.available()) 
  {
    char inChar = (char)Serial.read(); 
    GPSString += inChar;
    if (inChar == '\n') {
    stringComplete = true;
    }
  }
  if (stringComplete) 
  {    
    if(GPSString.indexOf("$GPGGA") != -1) // check for correct header
    {
    Serial.println(GPSString);
    
//----------------Latitude ----------------------------------------------------------------      
      GPSStringbuf = GPSString.substring(position(2), position(3) - 1); 
      lat = GPSStringbuf.substring(0,7);

//----------------NS---------------------------------------------------      
      GPSStringbuf = GPSString.substring(position(3), position(4) - 1); 
      ns = GPSStringbuf;

//----------------Longitude -------------------------------------------                  
      GPSStringbuf = GPSString.substring(position(4), position(5) - 1); 
      lon = GPSStringbuf.substring(0,8);
 
//----------------EW----------------------------------------------------      
      GPSStringbuf = GPSString.substring(position(5), position(6) - 1); 
      ew = GPSStringbuf;

//----------------Status------------------------------------------------
      GPSStringbuf = GPSString.substring(position(6), position(7) - 1); 
      GPSStringbuf.toCharArray(floatbuf, sizeof(floatbuf)); 
      status = atoi(floatbuf);     

//----------------Altitude ( feet ) ------------------------------------
      GPSStringbuf = GPSString.substring(position(9), position(10) - 1); 
      GPSStringbuf.toCharArray(floatbuf, sizeof(floatbuf)); 
      alti = (atoi(floatbuf))*2.54;     

// only sending valid data
   if(status>0) { 			//  DL1STL: evaluation changed here because GPS modules send 1 or 2 here. 
     
   digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
   delay(500);
// read the input on analog pin 0:
   int sensorValue = analogRead(A7);
  // calculate: 5V voltage reference of the Arduinos with 1:4 voltage divider 
  // Umrechnung: 5V Spannungsreferenz des Arduinos mit vorgeschalteten 1:4 Spannungsteler 
  // -> 20V reference with 10Bit ADC, that is 2¹⁰ as maximum
  float voltage = sensorValue * 20.0 / 1024;


// combining part strings for APRS; with output of Altitude ( feet )
//   aprs = "!" + lat + ns + "/" + lon + ew + ">A=" + alti + "#";

// output of temperature
   aprs = "!" + lat + ns + "/" + lon + ew + ">Temperatur=" + bmp.readTemperature() + "*C; VDD=" + voltage + "V; QRV:144,625MHz";

// convert string to char_array
   int str_len = aprs.length() + 1; 
   char char_array[str_len];
   aprs.toCharArray(char_array, str_len);

// send via packet_buffer 
   packet_buffer = char_array;
   Serial.print("Send:    "),Serial.println(packet_buffer); 
   QAPRS.send(from_addr, '0', dest_addr, '0', relays, packet_buffer);
   }

// variable delay depending on #A0 analog value 
   int val,z,calc;
   val = analogRead(A0);  // read analog value from poti ( 0 to 1023 )
   calc = map(val,0,1023,10,600);
   Serial.print("Delay:   "),Serial.print(calc),Serial.println(" Secs.");
   
//   Serial.print("Temperature = ");   
//   Serial.print(bmp.readTemperature());
//   Serial.println(" *C"), Serial.println(" ");

   digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW

   for(z=1; z<(calc); z++)  
 { delay(1000); }     // delay = 10-600 Secs. (10 Sec. =10 Min.)
   }
    GPSString = "";
    GPSStringbuf = "";
    stringComplete = false;
  }
}

/////////////////////////////////////////////////////////////////////////////////////////// 
int position(int number) 
{
  int i = 0;
  int position = 0;
  while(i < number)
  {
    position = GPSString.indexOf(",", position) + 1; 
    i++;
  }
  return position;
}


